# BenchExec is a framework for reliable benchmarking.
# This file is part of BenchExec.
#
# Copyright (C) 2007-2015  Dirk Beyer
# All rights reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import os
import logging
import subprocess

import benchexec.result as result
import benchexec.util as util
import benchexec.tools.template


from benchexec.tools.cpachecker import Tool as CPAchecker
from benchexec.tools.esbmc import Tool as ESBMC
from benchexec.tools.symbiotic import Tool as Symbiotic
from benchexec.tools.veriabs import Tool as VeriAbs

class Tool(benchexec.tools.template.BaseTool):

    REQUIRED_PATHS = ["conditional_verifiers_fold.sh"] +\
                ["CPAchecker/" + p for p in CPAchecker.REQUIRED_PATHS] +\
                ["cpa-seq/" + p for p in CPAchecker.REQUIRED_PATHS] +\
                ["esbmc/" + p for p in ESBMC.REQUIRED_PATHS] +\
                ["symbiotic/" + p for p in Symbiotic.REQUIRED_PATHS_7_0_0] +\
                ["VeriAbs/" + p for p in VeriAbs.REQUIRED_PATHS] 

    def executable(self):
        return util.find_executable('conditional_verifiers_fold.sh')


    def version(self, executable):
        return self._version_from_tool(executable)

    def name(self):
        return 'CMCFoldReducer'


    def cmdline(self, executable, options, tasks, propertyfile=None, rlimits={}):

        assert len(tasks) == 1
        assert propertyfile is not None

        option_list = []
        for option in options:
            option_list = option_list + option.split(";")

        return [executable] + option_list + [propertyfile] + tasks


    def determine_result(self, returncode, returnsignal, output, isTimeout):
        status = result.RESULT_UNKNOWN

        for line in output:
            index = line.find('CMC verification result:')
            if index != -1:
                if  line[index+24:].strip() == 'CORRECT':
                    status = result.RESULT_TRUE_PROP
                elif  line[index+24:].strip() == 'VIOLATION':
                    status = result.RESULT_FALSE_REACH
                break

        return status

